@@ -10,11 +10,11 @@
 #'
 #' ## Binary Treatments
 #'
-#' For binary treatments, this method estimates the weights using `osqp()` using formulas described by Huling and Mak (2022). The following estimands are allowed: ATE, ATT, and ATC.
+#' For binary treatments, this method estimates the weights using `osqp()` using formulas described by Huling and Mak (2024). The following estimands are allowed: ATE, ATT, and ATC.
 #'
 #' ## Multi-Category Treatments
 #'
-#' For multi-category treatments, this method estimates the weights using `osqp()` using formulas described by Huling and Mak (2022). The following estimands are allowed: ATE and ATT.
+#' For multi-category treatments, this method estimates the weights using `osqp()` using formulas described by Huling and Mak (2024). The following estimands are allowed: ATE and ATT.
 #'
 #' ## Continuous Treatments
 #'
@@ -46,12 +46,12 @@
 #' \describe{
 #'   \item{`dist.mat`}{the name of the method used to compute the distance matrix of the covariates or the numeric distance matrix itself. Allowable options include `"scaled_euclidean"` for the Euclidean (L2) distance on the scaled covariates (the default), `"mahalanobis"` for the Mahalanobis distance, and `"euclidean"` for the raw Euclidean distance. Abbreviations allowed. Note that some user-supplied distance matrices can cause the R session to abort due to a bug within \pkg{osqp}, so this argument should be used with caution. A distance matrix must be a square, symmetric, numeric matrix with zeros along the diagonal and a row and column for each unit. Can also be supplied as the output of a call to [dist()].
 #'   }
-#'   \item{`lambda`}{a positive numeric scalar used to penalize the square of the weights. This value divided by the square of the total sample size is added to the diagonal of the quadratic part of the loss function. Higher values favor weights with less variability. Note this is distinct from the lambda value described in Huling and Mak (2022), which penalizes the complexity of individual treatment rules rather than the weights, but does correspond to lambda from Huling et al. (2021). Default is .0001, which is essentially 0.
+#'   \item{`lambda`}{a positive numeric scalar used to penalize the square of the weights. This value divided by the square of the total sample size is added to the diagonal of the quadratic part of the loss function. Higher values favor weights with less variability. Note this is distinct from the lambda value described in Huling and Mak (2024), which penalizes the complexity of individual treatment rules rather than the weights, but does correspond to lambda from Huling et al. (2023). Default is .0001, which is essentially 0.
 #'   }
 #' }
 #' For binary and multi-category treatments, the following additional arguments can be specified:
 #'   \describe{
-#'     \item{`improved`}{`logical`; whether to use the improved energy balancing weights as described by Huling and Mak (2022) when `estimand = "ATE"`. This involves optimizing balance not only between each treatment group and the overall sample, but also between each pair of treatment groups. Huling and Mak (2022) found that the improved energy balancing weights generally outperformed standard energy balancing. Default is `TRUE`; set to `FALSE` to use the standard energy balancing weights instead (not recommended).
+#'     \item{`improved`}{`logical`; whether to use the improved energy balancing weights as described by Huling and Mak (2024) when `estimand = "ATE"`. This involves optimizing balance not only between each treatment group and the overall sample, but also between each pair of treatment groups. Huling and Mak (2024) found that the improved energy balancing weights generally outperformed standard energy balancing. Default is `TRUE`; set to `FALSE` to use the standard energy balancing weights instead (not recommended).
 #'     }
 #'   \item{`quantile`}{
 #'     A named list of quantiles (values between 0 and 1) for each continuous covariate, which are used to create additional variables that when balanced ensure balance on the corresponding quantile of the variable. For example, setting `quantile = list(x1 = c(.25, .5. , .75))` ensures the 25th, 50th, and 75th percentiles of `x1` in each treatment group will be balanced in the weighted sample. Can also be a single number (e.g., `.5`) or an unnamed list of length 1 (e.g., `list(c(.25, .5, .75))`) to request the same quantile(s) for all continuous covariates, or a named vector (e.g., `c(x1 = .5, x2 = .75`) to request one quantile for each covariate.
@@ -64,11 +64,11 @@
 #'       The number of moments of the treatment and covariate distributions that are constrained to be the same in the weighted sample as in the original sample. For example, setting `d.moments = 3` ensures that the mean, variance, and skew of the treatment and covariates are the same in the weighted sample as in the unweighted sample. `d.moments` should be greater than or equal to `moments` and will be automatically set accordingly if not (or if not specified).
 #'     }
 #'     \item{`dimension.adj`}{
-#'       `logical`; whether to include the dimensionality adjustment described by Huling et al. (2021). If `TRUE`, the default, the energy distance for the covariates is weighted \eqn{\sqrt{p}} times as much as the energy distance for the treatment, where \eqn{p} is the number of covariates. If `FALSE`, the two energy distances are given equal weights. Default is `TRUE`.
+#'       `logical`; whether to include the dimensionality adjustment described by Huling et al. (2023). If `TRUE`, the default, the energy distance for the covariates is weighted \eqn{\sqrt{p}} times as much as the energy distance for the treatment, where \eqn{p} is the number of covariates. If `FALSE`, the two energy distances are given equal weights. Default is `TRUE`.
 #'     }
 #'   }
 #'
-#' The `moments` argument functions differently for `method = "energy"` from how it does with other methods. When unspecified or set to zero, energy balancing weights are estimated as described by Huling and Mak (2022) for binary and multi-category treatments or by Huling et al. (2023) for continuous treatments. When `moments` is set to an integer larger than 0, additional balance constraints on the requested moments of the covariates are also included, guaranteeing exact moment balance on these covariates while minimizing the energy distance of the weighted sample. For binary and multi-category treatments, this involves exact balance on the means of the entered covariates; for continuous treatments, this involves exact balance on the treatment-covariate correlations of the entered covariates.
+#' The `moments` argument functions differently for `method = "energy"` from how it does with other methods. When unspecified or set to zero, energy balancing weights are estimated as described by Huling and Mak (2024) for binary and multi-category treatments or by Huling et al. (2023) for continuous treatments. When `moments` is set to an integer larger than 0, additional balance constraints on the requested moments of the covariates are also included, guaranteeing exact moment balance on these covariates while minimizing the energy distance of the weighted sample. For binary and multi-category treatments, this involves exact balance on the means of the entered covariates; for continuous treatments, this involves exact balance on the treatment-covariate correlations of the entered covariates.
 #'
 #' Any other arguments will be passed to \pkgfun{osqp}{osqpSettings}. Some defaults differ from those in `osqpSettings()`; see *Reproducibility* below.
 #'
@@ -105,7 +105,7 @@
 #' @references
 #' ## Binary and multi-category treatments
 #'
-#' Huling, J. D., & Mak, S. (2022). Energy Balancing of Covariate Distributions (arXiv:2004.13962). arXiv. \doi{10.48550/arXiv.2004.13962}
+#' Huling, J. D., & Mak, S. (2024). Energy balancing of covariate distributions. *Journal of Causal Inference*, 12(1). \doi{10.1515/jci-2022-0029}
 #'
 #' ## Continuous treatments
 #'
@@ -162,7 +162,8 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
     if (inherits(d, "dist")) d <- as.matrix(d)
 
     if (!is.matrix(d) || !all(dim(d) == length(treat)) ||
-        !all(check_if_zero(diag(d))) || any(d < 0) ||
+        !all(check_if_zero(diag(d))) ||
+        any(d < 0) ||
         !isSymmetric(unname(d))) {
       .err(sprintf("`dist.mat` must be one of %s or a square, symmetric distance matrix with a value for all pairs of units",
                    word_list(weightit_distances(), "or", quotes = TRUE)))
@@ -199,7 +200,7 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
   n0 <- length(t0)
   n1 <- length(t1)
 
-  s.weights_n_0 <- s.weights_n_1 <- rep(0, n)
+  s.weights_n_0 <- s.weights_n_1 <- rep.int(0, n)
   s.weights_n_0[t0] <- s.weights[t0] / n0
   s.weights_n_1[t1] <- s.weights[t1] / n1
 
@@ -219,7 +220,7 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
 
     #Constraints for positivity and sum of weights
     Amat <- cbind(diagn, s.weights_n_0, s.weights_n_1)
-    lvec <- c(rep(min.w, n), 1, 1)
+    lvec <- c(rep.int(min.w, n), 1, 1)
     uvec <- c(ifelse(check_if_zero(s.weights), min.w, Inf), 1, 1)
 
     if (moments != 0 || int) {
@@ -243,6 +244,36 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
       uvec <- c(uvec, targets, targets)
     }
   }
+  else if (estimand == "ATO") {
+
+    nn <- tcrossprod(s.weights_n_0 - s.weights_n_1)
+
+    P <- -d * nn
+
+    q <- NULL
+
+    #Constraints for positivity and sum of weights
+    Amat <- cbind(diagn, s.weights_n_0, s.weights_n_1)
+    lvec <- c(rep.int(min.w, n), 1, 1)
+    uvec <- c(ifelse(check_if_zero(s.weights), min.w, Inf), 1, 1)
+
+    if (moments != 0 || int) {
+      #Exactly balance moments and/or interactions
+      covs <- cbind(covs, .int_poly_f(covs, poly = moments, int = int))
+
+      Amat <- cbind(Amat, covs * (s.weights_n_0 - s.weights_n_1))
+      lvec <- c(lvec, rep.int(0, ncol(covs)))
+      uvec <- c(uvec, rep.int(0, ncol(covs)))
+    }
+
+    if (is_not_null(A[["quantile"]])) {
+      qu <- .quantile_f(covs, qu = A[["quantile"]], s.weights = s.weights)
+
+      Amat <- cbind(Amat, qu * (s.weights_n_0 - s.weights_n_1))
+      lvec <- c(lvec, rep.int(0, ncol(qu)))
+      uvec <- c(uvec, rep.int(0, ncol(qu)))
+    }
+  }
   else if (estimand == "ATT") {
     nn <- tcrossprod(s.weights_n_0[t0])
 
@@ -251,7 +282,7 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
     q <- 2 * (s.weights_n_1[t1] %*% d[t1, t0]) * s.weights_n_0[t0]
 
     Amat <- cbind(diag(n0), s.weights_n_0[t0])
-    lvec <- c(rep(min.w, n0), 1)
+    lvec <- c(rep.int(min.w, n0), 1)
     uvec <- c(ifelse(check_if_zero(s.weights[t0]), min.w, Inf), 1)
 
     if (moments != 0 || int) {
@@ -286,7 +317,7 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
     q <- 2 * (s.weights_n_0[t0] %*% d[t0, t1]) * s.weights_n_1[t1]
 
     Amat <- cbind(diag(n1), s.weights_n_1[t1])
-    lvec <- c(rep(min.w, n1), 1)
+    lvec <- c(rep.int(min.w, n1), 1)
     uvec <- c(ifelse(check_if_zero(s.weights[t1]), min.w, Inf), 1)
 
     if (moments != 0 || int) {
@@ -365,17 +396,17 @@ weightit2energy <- function(covs, treat, s.weights, subset, estimand, focal,
   }
 
 
-  if (estimand == "ATE") {
-    w <- opt.out$x
-  }
-  else if (estimand == "ATT") {
-    w <- rep(1, n)
+  if (estimand == "ATT") {
+    w <- rep.int(1, n)
     w[t0] <- opt.out$x
   }
   else if (estimand == "ATC") {
-    w <- rep(1, n)
+    w <- rep.int(1, n)
     w[t1] <- opt.out$x
   }
+  else {
+    w <- opt.out$x
+  }
 
   w[w <= min.w] <- min.w
 
@@ -461,8 +492,8 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
 
     #Constraints for positivity and sum of weights
     Amat <- cbind(diagn, s.weights_n_t)
-    lvec <- c(rep(min.w, n), rep(1, length(levels_treat)))
-    uvec <- c(ifelse(check_if_zero(s.weights), min.w, Inf), rep(1, length(levels_treat)))
+    lvec <- c(rep.int(min.w, n), rep.int(1, length(levels_treat)))
+    uvec <- c(ifelse(check_if_zero(s.weights), min.w, Inf), rep.int(1, length(levels_treat)))
 
     if (moments != 0 || int) {
       #Exactly balance moments and/or interactions
@@ -471,8 +502,8 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
       targets <- col.w.m(covs, s.weights)
 
       Amat <- cbind(Amat, do.call("cbind", apply(s.weights_n_t, 2, function(x) covs * x, simplify = FALSE)))
-      lvec <- c(lvec, rep(targets, length(levels_treat)))
-      uvec <- c(uvec, rep(targets, length(levels_treat)))
+      lvec <- c(lvec, rep.int(targets, length(levels_treat)))
+      uvec <- c(uvec, rep.int(targets, length(levels_treat)))
     }
 
     if (is_not_null(A[["quantile"]])) {
@@ -481,8 +512,8 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
       targets <- col.w.m(qu, s.weights)
 
       Amat <- cbind(Amat, do.call("cbind", apply(s.weights_n_t, 2, function(x) qu * x, simplify = FALSE)))
-      lvec <- c(lvec, rep(targets, length(levels_treat)))
-      uvec <- c(uvec, rep(targets, length(levels_treat)))
+      lvec <- c(lvec, rep.int(targets, length(levels_treat)))
+      uvec <- c(uvec, rep.int(targets, length(levels_treat)))
     }
   }
   else {
@@ -497,8 +528,8 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
       rowSums(s.weights_n_t[!in_focal, non_focal, drop = FALSE])
 
     Amat <- cbind(diag(sum(!in_focal)), s.weights_n_t[!in_focal, non_focal])
-    lvec <- c(rep(min.w, sum(!in_focal)), rep(1, length(non_focal)))
-    uvec <- c(ifelse_(check_if_zero(s.weights[!in_focal]), min.w, Inf), rep(1, length(non_focal)))
+    lvec <- c(rep.int(min.w, sum(!in_focal)), rep.int(1, length(non_focal)))
+    uvec <- c(ifelse(check_if_zero(s.weights[!in_focal]), min.w, Inf), rep.int(1, length(non_focal)))
 
     if (moments != 0 || int) {
       #Exactly balance moments and/or interactions
@@ -509,8 +540,8 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
       Amat <- cbind(Amat, do.call("cbind", apply(s.weights_n_t[!in_focal, non_focal, drop = FALSE], 2,
                                                  function(x) covs[!in_focal,, drop = FALSE] * x,
                                                  simplify = FALSE)))
-      lvec <- c(lvec, rep(targets, length(non_focal)))
-      uvec <- c(uvec, rep(targets, length(non_focal)))
+      lvec <- c(lvec, rep.int(targets, length(non_focal)))
+      uvec <- c(uvec, rep.int(targets, length(non_focal)))
     }
 
     if (is_not_null(A[["quantile"]])) {
@@ -522,8 +553,8 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
       Amat <- cbind(Amat, do.call("cbind", apply(s.weights_n_t[!in_focal, non_focal, drop = FALSE], 2,
                                                  function(x) qu[!in_focal,, drop = FALSE] * x,
                                                  simplify = FALSE)))
-      lvec <- c(lvec, rep(targets, length(non_focal)))
-      uvec <- c(uvec, rep(targets, length(non_focal)))
+      lvec <- c(lvec, rep.int(targets, length(non_focal)))
+      uvec <- c(uvec, rep.int(targets, length(non_focal)))
     }
   }
 
@@ -581,7 +612,7 @@ weightit2energy.multi <- function(covs, treat, s.weights, subset, estimand, foca
     w <- opt.out$x
   }
   else {
-    w <- rep(1, n)
+    w <- rep.int(1, n)
     w[treat != focal] <- opt.out$x
   }
 
@@ -700,7 +731,7 @@ weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, moment
   q <- q * s.weights
 
   Amat <- cbind(diag(n), s.weights)
-  lvec <- c(rep(min.w, n), n)
+  lvec <- c(rep.int(min.w, n), n)
   uvec <- c(ifelse(sw0, min.w, Inf), n)
 
   if (d.moments != 0) {
@@ -718,8 +749,8 @@ weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, moment
     d.treat <- scale(d.treat, center = A.targets, scale = FALSE)
 
     Amat <- cbind(Amat, d.covs * s.weights, d.treat * s.weights)
-    lvec <- c(lvec, rep(0, ncol(d.covs)), rep(0, ncol(d.treat)))
-    uvec <- c(uvec, rep(0, ncol(d.covs)), rep(0, ncol(d.treat)))
+    lvec <- c(lvec, rep.int(0, ncol(d.covs)), rep.int(0, ncol(d.treat)))
+    uvec <- c(uvec, rep.int(0, ncol(d.covs)), rep.int(0, ncol(d.treat)))
   }
 
   if (moments != 0 || int) {
@@ -733,8 +764,8 @@ weightit2energy.cont <- function(covs, treat, s.weights, subset, missing, moment
 
     Amat <- cbind(Amat, covs * treat * s.weights)
 
-    lvec <- c(lvec, rep(0, ncol(covs)))
-    uvec <- c(uvec, rep(0, ncol(covs)))
+    lvec <- c(lvec, rep.int(0, ncol(covs)))
+    uvec <- c(uvec, rep.int(0, ncol(covs)))
   }
 
   #Add weight penalty
